import React, { useState } from 'react';
import Modal from '@material-ui/core/Modal';
import classNames from 'classnames';
import Button from '@material-ui/core/Button';
import Typography from '@material-ui/core/Typography';
import Paper from '@material-ui/core/Paper';
import TextField from '@material-ui/core/TextField';
import { withStyles } from '@material-ui/core/styles';
import ErrorIcon from '@material-ui/icons/Error';
import Snackbar from '@material-ui/core/Snackbar';
import SnackbarContent from '@material-ui/core/SnackbarContent';
import IconButton from '@material-ui/core/IconButton';
import CloseIcon from '@material-ui/icons/Close';

const variantIcon = {
  // success:  CheckCircleIcon,
  // warning: WarningIcon,
  error: ErrorIcon,
  // info: InfoIcon,
};

const styles = theme => ({
  paper: {
    position: 'absolute',
    width: theme.spacing.unit * 50,
    backgroundColor: theme.palette.background.paper,
    boxShadow: theme.shadows[5],
    padding: theme.spacing.unit * 2,
  },
  container: {
    display: 'flex',
    flexWrap: 'wrap',
  },
  textField: {
    marginLeft: theme.spacing.unit,
    marginRight: theme.spacing.unit,
    width: '-webkit-fill-available'
  },
  menu: {
    width: 200,
  },
  // success: {
  //   backgroundColor: green[600],
  // },
  error: {
    backgroundColor: theme.palette.error.dark,
  },
  // info: {
  //   backgroundColor: theme.palette.primary.dark,
  // },
  // warning: {
  //   backgroundColor: amber[700],
  // },
  icon: {
    fontSize: 20,
  },
  iconVariant: {
    opacity: 0.9,
    marginRight: theme.spacing.unit,
  },
  message: {
    display: 'flex',
    alignItems: 'center',
  },
});

function MySnackbarContent(props) {
  const { classes, className, message, onClose, variant, ...other } = props;
  const Icon = variantIcon[variant];

  return (
    <SnackbarContent
      className={classNames(classes[variant], className)}
      aria-describedby="client-snackbar"
      message={
        <span id="client-snackbar" className={classes.message}>
          <Icon className={classNames(classes.icon, classes.iconVariant)} />
          {message}
        </span>
      }
      action={[
        <IconButton
          key="close"
          aria-label="Close"
          color="inherit"
          className={classes.close}
          onClick={onClose}
        >
          <CloseIcon className={classes.icon} />
        </IconButton>,
      ]}
      {...other}
    />
  );
}

const MySnackbarContentWrapper = withStyles(styles)(MySnackbarContent);

const AddWaitingTaskModal = ({ classes, handleSubmitTaskData }) => {

  const [isModalOpen, setIsModalOpen] = useState(false);
  const [snackbarOpen, setSnackbarOpen] = useState(false)
  const [taskData, setTaskData] = useState({
    Subject: '',
    Remarks: '',
  })

  const submitTask = () => {
    if(taskData.Subject === '' || taskData.Remarks === '') {
      setSnackbarOpen(true);
    } else {
      handleSubmitTaskData(taskData);
      setIsModalOpen(false);
      setTaskData({
        Subject: '',
        Remarks: '',
      });
    }
  }

  return (
    <React.Fragment>
      <Button variant="contained" color="primary" onClick={() => setIsModalOpen(true)}>
        Add Task in Waiting
      </Button>
      <Modal
        aria-labelledby="simple-modal-title"
        aria-describedby="simple-modal-description"
        open={isModalOpen}
        onClose={() => setIsModalOpen(false)}
        style={{display:'flex',alignItems:'center',justifyContent:'center'}}
      >
        <Paper className={classes.paper}>
          <Typography variant="title" id="modal-title">
            <strong>Add Waiting Task</strong>
          </Typography>
          <TextField
            label="Title"
            className={classes.textField}
            value={taskData.Subject}
            onChange={e => setTaskData({...taskData, Subject: e.target.value})}
            margin="normal"
          />
          <TextField
            label="Remarks"
            className={classes.textField}
            value={taskData.Remarks}
            onChange={e => setTaskData({...taskData, Remarks: e.target.value})}
            margin="normal"
          />
          <div style={{ display: 'flex', justifyContent: 'flex-end', marginTop: 20 }}>
            <Button variant="contained" color="primary" onClick={submitTask}>
              Save
            </Button>
            <Button color="primary" style={{ marginLeft: 10 }} onClick={() => setIsModalOpen(false)}>
              Cancel
            </Button>
          </div>
        <Snackbar
          anchorOrigin={{
            vertical: 'bottom',
            horizontal: 'center',
          }}
          open={snackbarOpen}
          autoHideDuration={6000}
          onClose={() => setSnackbarOpen(false)}
        >
          <MySnackbarContentWrapper
            onClose={() => setSnackbarOpen(false)}
            variant="error"
            message="Please fill all the fields"
          />
        </Snackbar>
        </Paper>
      </Modal>
    </React.Fragment>
  )
}

export default withStyles(styles)(AddWaitingTaskModal);
