import React, { useState } from 'react';
import { Grid } from '@material-ui/core';
import { DateTimePickerComponent, DatePickerComponent } from '@syncfusion/ej2-react-calendars';
import { DropDownListComponent, MultiSelectComponent } from '@syncfusion/ej2-react-dropdowns';
import { TextBoxComponent, ColorPickerComponent } from '@syncfusion/ej2-react-inputs';
import { CheckBoxComponent } from '@syncfusion/ej2-react-buttons';
import { RecurrenceEditorComponent } from '@syncfusion/ej2-react-schedule';

const EventEditorTemplate = (props) => {
  const [ isTimeZone, setIsTimeZone ] = useState(props.Timezone || false);
  const [isAllDay, setIsAllDay] = useState(props.AllDay || false);

  let Datecomponent = DateTimePickerComponent;
  let format = 'dd/MM/yy hh:mm a';
  if(isAllDay) {
    Datecomponent = DatePickerComponent;
    format = 'dd/MM/yy';
  }
  return (
    <div style={{ padding: '10px 15px 0 15px' }}>
      <Grid style={{ paddingTop: 15 }} container spacing={24}>
        <Grid item xs={12}>
          <TextBoxComponent
            name="Subject"
            data-name="Subject"
            id="Subject"
            className="e-field e-input"
            placeholder="Title"
            floatLabelType="Auto"
            value={props.Subject}
            autofocus={true}
          />
        </Grid>
        <Grid item xs={6}>
          <Datecomponent
            id="StartTime"
            format={format}
            data-name="StartTime"
            value={new Date(props.StartTime)}
            className="e-field"
          />
        </Grid>
        <Grid item xs={6}>
          <Datecomponent
            id="EndTime"
            format={format}
            data-name="EndTime"
            value={new Date(props.EndTime)}
            className="e-field"
          />
        </Grid>
        <Grid item xs={6}>
          <TextBoxComponent
            name="StartLocation"
            data-name="StartLocation"
            id="StartLocation"
            className="e-field e-input"
            placeholder="Start Location"
            floatLabelType="Auto"
            value={props.StartLocation || ''}
          />
        </Grid>
        <Grid item xs={6}>
          <TextBoxComponent
            name="EndLocation"
            data-name="EndLocation"
            id="EndLocation"
            className="e-field e-input"
            placeholder="End Location"
            floatLabelType="Auto"
            value={props.EndLocation || ''}
          />
        </Grid>
        <Grid
          item
          xs={12}
          style={{
            display: 'flex',
            alignItems: 'center',
            justifyContent: 'space-around'
          }}
        >
          <CheckBoxComponent
            name="AllDay"
            data-name="AllDay"
            id="AllDay"
            className="e-field"
            checked={props.AllDay}
            onClick={(e) => setIsAllDay(e.target.checked)}
            label="All Day?"
          />
          <CheckBoxComponent
            name="Timezone"
            data-name="Timezone"
            className="e-field"
            cssClass={`e-checkbox-wrapper ${!isAllDay ? 'e-enable' : 'e-disable'}`}
            // cssClass={`e-checkbox-wrapper e-wrapper e-time-zone`}
            checked={props.Timezone}
            onClick={(e) => setIsTimeZone(e.target.checked)}
            label="Timezone"
            title="Timezone"
          />
          <ColorPickerComponent
            id="Color"
            data-name="Color"
            className="e-field"
            mode={'Palette'}
            disabled={false}
            showButtons={true}
            value={props.CategoryColor || '#5ab9de'}
            modeSwitcher={'true'}
            // change={props.onChange.bind(this)}
          />
        </Grid>
        <div className={`e-time-zone-row ${isTimeZone && !isAllDay ? 'e-enable' : ''}`} xs={12} style={{ marginBottom: 0, padding: isTimeZone && !isAllDay ? 12 : 0 }}>
          <Grid className="e-start-time-zone-container" item xs={6}>
            <DropDownListComponent
              id="StartTimezone"
              placeholder="Start Timezone"
              data-name="StartTimezone"
              className="e-field"
              style={{ width: '100%' }}
              allowFiltering={true}
              dataSource={props.timezoneData}
              fields={{ text: 'Label', value: 'Value' }}
              filtering={props.onFilteringTimezone}
              value={props.StartTimezone || ''}
            />
          </Grid>
          <Grid className="e-end-time-zone-container" item xs={6}>
            <DropDownListComponent
              id="EndTimezone"
              placeholder="End Timezone"
              data-name="EndTimezone"
              className="e-field"
              style={{ width: '100%' }}
              filterBarPlaceholder="Search a timezone"
              allowFiltering={true}
              dataSource={props.timezoneData}
              fields={{ text: 'Label', value: 'Value' }}
              filtering={props.onFilteringTimezone}
              value={props.EndTimezone || ''}
            />
          </Grid>
        </div>
        <Grid item xs={12}>
          <RecurrenceEditorComponent
            id="RecurrenceRule"
            data-name="RecurrenceRule"
            name="RecurrenceRule"
            className="e-field"
            value={props.RecurrenceRule || ''}
          />
        </Grid>
        <Grid item xs={12}>
          <MultiSelectComponent
            id="EmployeeId"
            data-id="EmployeeId"
            className="e-field"
            dataSource={props.employeeData}
            fields={props.fields}
            mode="Box"
            placeholder="Select employee"
            itemTemplate={props.itemTemplate}
            valueTemplate={props.valueTemplate}
            value={[ props.EmployeeId ] || []}
            // headerTemplate={props.headerTemplate}
          />
        </Grid>
        <Grid item xs={6}>
          <DropDownListComponent
            id="Status"
            placeholder="Choose status"
            data-name="Status"
            className="e-field"
            style={{ width: '100%' }}
            dataSource={[ 'New', 'Start', 'Requested', 'Confirmed', 'Completed' ]}
            value={props.Status || ''}
          />
        </Grid>
      </Grid>
      <Grid container spacing={24}>
        <Grid item xs={6}>
          <TextBoxComponent
            name="Flight"
            data-name="Flight"
            id="Flight"
            className="e-field e-input"
            placeholder="Flight"
            floatLabelType="Auto"
            value={props.Flight || ''}
          />
        </Grid>
      </Grid>
      <button type="hidden" title="submit" style={{ display: 'none' }}></button>
    </div>
  );
};

export default EventEditorTemplate;
