import React, { createElement } from 'react';
import {
  ScheduleComponent,
  ResourcesDirective,
  ResourceDirective,
  ViewsDirective,
  ViewDirective,
  Inject,
  TimelineViews,
  Resize,
  DragAndDrop,
  TimelineMonth,
  Day,
  Week,
  RecurrenceEditorComponent
} from '@syncfusion/ej2-react-schedule';
import { DateTimePickerComponent, DatePickerComponent } from '@syncfusion/ej2-react-calendars';
import { DropDownListComponent, MultiSelectComponent } from '@syncfusion/ej2-react-dropdowns';
import { TextBoxComponent, ColorPickerComponent } from '@syncfusion/ej2-react-inputs';
import { CheckBoxComponent } from '@syncfusion/ej2-react-buttons';
import { TabComponent, TabItemDirective, TabItemsDirective, TreeViewComponent } from '@syncfusion/ej2-react-navigations';
import { addClass, removeClass, Browser, closest, remove, L10n } from '@syncfusion/ej2-base';
import {
  RichTextEditorComponent,
  Toolbar,
  Image,
  Link,
  HtmlEditor,
  Count,
  QuickToolbar,
  Table
} from '@syncfusion/ej2-react-richtexteditor';
import { DataManager, UrlAdaptor, Query } from '@syncfusion/ej2-data';

import * as CodeMirror from 'codemirror';
import 'codemirror/mode/javascript/javascript';
import 'codemirror/mode/css/css.js';
import 'codemirror/mode/htmlmixed/htmlmixed.js';

import Paper from '@material-ui/core/Paper';
import Grid from '@material-ui/core/Grid';

import { SampleBase } from './sample-base';
import dataSource from './dataSource.json';
import timezones from './timezones.json';
import './taskscheduler.css';
import { apiUrl } from '../../../App';
import EventEditorTemplate from './EventEditorTemplate';
import AddWaitingTaskModal from './AddWaitingTaskModal';

L10n.load({
  'en-US': {
    'schedule': {
      'saveButton': 'Add',
      'cancelButton': 'Close',
      'deleteButton': 'Remove',
      'newEvent': 'Add Event'
    }
  }
});

class TaskScheduler extends SampleBase {
  constructor() {
    super(...arguments);

    // this.data = extend([], this.state.tasks, null, true);

    this.employeeData = [
      { Text: 'Alice', Id: 1, GroupId: 1, Color: '#bbdc00', Designation: 'Content writer', UserImage: 'https://ej2.syncfusion.com/react/demos/src/schedule/images/alice.png' },
      { Text: 'Nancy', Id: 2, GroupId: 2, Color: '#9e5fff', Designation: 'Designer', UserImage: 'https://ej2.syncfusion.com/react/demos/src/schedule/images/robert.png' },
      { Text: 'Robert', Id: 3, GroupId: 1, Color: '#bbdc00', Designation: 'Software Engineer', UserImage: 'https://ej2.syncfusion.com/react/demos/src/schedule/images/robson.png' },
      { Text: 'Robson', Id: 4, GroupId: 2, Color: '#9e5fff', Designation: 'Support Engineer', UserImage: 'https://ej2.syncfusion.com/react/demos/src/schedule/images/laura.png' },
      { Text: 'Laura', Id: 5, GroupId: 1, Color: '#bbdc00', Designation: 'Human Resource', UserImage: 'https://ej2.syncfusion.com/react/demos/src/schedule/images/nancy.png' },
      { Text: 'Margaret', Id: 6, GroupId: 2, Color: '#9e5fff', Designation: 'Content Analyst', UserImage: 'https://ej2.syncfusion.com/react/demos/src/schedule/images/margaret.png' }
    ];

    this.allowDragAndDrops = true;
    this.isTreeItemDropped = false;
    this.draggedItemId = '';
    console.log('dataSource', dataSource);

    // RichTextEditor items list
    this.items = [
      'Bold',
      'Italic',
      'Underline',
      'StrikeThrough',
      'FontName',
      'FontSize',
      'FontColor',
      'BackgroundColor',
      'LowerCase',
      'UpperCase',
      '|',
      'Formats',
      'Alignments',
      'OrderedList',
      'UnorderedList',
      'Outdent',
      'Indent',
      'SuperScript',
      'SubScript',
      '|',
      'CreateTable',
      'CreateLink',
      'Image',
      '|',
      'ClearFormat',
      'Print',
      'SourceCode',
      'FullScreen',
      '|',
      'Undo',
      'Redo'
    ];

    //RichTextEditor ToolbarSettings
    this.toolbarSettings = {
      items: this.items
    };

    this.dataManger = new DataManager({
      url: `${apiUrl}/tasks`,
      adaptor: new UrlAdaptor,
      crossDomain: true
    });

    this.fields = { text: 'Text', value: 'Id' };

    this.timezoneData = timezones;

    this.onFilteringTimezone = (e) => {
      let query = new Query();
      //frame the query based on search string with filter type.
      query = (e.text !== '') ? query.where('Label', 'startswith', e.text, true) : query;
      //pass the filter data source, filter query to updateData method.
      e.updateData(this.timezoneData, query);
    };

    this.treeViewFields = { dataSource: dataSource.waitingList, id: 'Id', text: 'Subject', expanded: 'expanded' };

    this.treeObj = React.createRef();
    this.scheduleObj = React.createRef();
  }

  getEmployeeName(value) {
    return value.resourceData[value.resource.textField];
  }

  getEmployeeImage(value) {
    let resourceName = this.getEmployeeName(value);
    return resourceName.toLowerCase();
  }

  getEmployeeDesignation(value) {
    return value.resourceData.Designation;
  }

  mirrorConversion = (e) => {
    this.textArea = this.rteObj.contentModule.getEditPanel();
    let id = this.rteObj.getID() + 'mirror-view';
    let mirrorView = this.rteObj.element.querySelector('#' + id);
    let charCount = this.rteObj.element.querySelector('.e-rte-character-count');
    if (e.targetItem === 'Preview') {
      this.textArea.style.display = 'block';
      mirrorView.style.display = 'none';
      this.textArea.innerHTML = this.myCodeMirror.getValue();
      charCount.style.display = 'block';
    } else {
      if (!mirrorView) {
        mirrorView = createElement('div', { className: 'e-content' });
        mirrorView.id = id;
        this.textArea.parentNode.appendChild(mirrorView);
      } else {
        mirrorView.innerHTML = '';
      }
      this.textArea.style.display = 'none';
      mirrorView.style.display = 'block';
      this.renderCodeMirror(mirrorView, this.rteObj.value);
      charCount.style.display = 'none';
    }
  };

  renderCodeMirror = (mirrorView, content) => {
    this.myCodeMirror = CodeMirror(mirrorView, {
      value: content,
      lineNumbers: true,
      mode: 'text/html',
      lineWrapping: true
    });
  };

  handleFullScreen = (e) => {
    let sbCntEle = document.querySelector('.sb-content.e-view');
    let sbHdrEle = document.querySelector('.sb-header.e-view');
    let leftBar;
    let transformElement;
    if (Browser.isDevice) {
      leftBar = document.querySelector('#right-sidebar');
      transformElement = document.querySelector('.sample-browser.e-view.e-content-animation');
    } else {
      leftBar = document.querySelector('#left-sidebar');
      transformElement = document.querySelector('#right-pane');
    }
    if (e.targetItem === 'Maximize') {
      if (Browser.isDevice && Browser.isIos) {
        addClass([ sbCntEle, sbHdrEle ], [ 'hide-header' ]);
      }
      addClass([ leftBar ], [ 'e-close' ]);
      removeClass([ leftBar ], [ 'e-open' ]);
      if (!Browser.isDevice) {
        transformElement.style.marginLeft = '0px';
      }
      transformElement.style.transform = 'inherit';
    } else if (e.targetItem === 'Minimize') {
      if (Browser.isDevice && Browser.isIos) {
        removeClass([ sbCntEle, sbHdrEle ], [ 'hide-header' ]);
      }
      removeClass([ leftBar ], [ 'e-close' ]);
      if (!Browser.isDevice) {
        addClass([ leftBar ], [ 'e-open' ]);
        transformElement.style.marginLeft = leftBar.offsetWidth + 'px';
      }
      transformElement.style.transform = 'translateX(0px)';
    }
  };

  actionCompleteHandler = (e) => {
    if (e.targetItem && (e.targetItem === 'SourceCode' || e.targetItem === 'Preview')) {
      this.rteObj.sourceCodeModule.getPanel().style.display = 'none';
      this.mirrorConversion(e);
    } else {
      setTimeout(() => {
        this.rteObj.toolbarModule.refreshToolbarOverflow();
      }, 400);
    }
  };

  resourceHeaderTemplate = (props) => {
    return (
      <div className="template-wrap">
        <div className="employee-category">
          <div className={'employee-image ' + this.getEmployeeImage(props)} />
          <div className="employee-name">{this.getEmployeeName(props)}</div>
          <div className="employee-designation">{this.getEmployeeDesignation(props)}</div>
        </div>
      </div>
    );
  }

  //set the value to item template
  itemTemplate = (data) => {
    return (
      <div style={{ display: 'flex' }}>
        <img className="empImage" src={data.UserImage} alt="employee" />
        <div>
          <div className="ms-ename"> {data.Text} </div>
          <div className="ms-job"> {data.Designation} </div>
        </div>
      </div>
    );
  }

  valueTemplate = (data) => {
    return <span> {data.Text} </span>;
  }

  headertext = [ { text: 'General' }, { text: 'Dependency' }, { text: 'Resources' }, { text: 'Notes' } ];

  editGeneralDataTab = (props) => {
    console.log('props', props);
    let Datecomponent = DateTimePickerComponent;
    let format = 'dd/MM/yy hh:mm a';
    if(props.AllDay) {
      Datecomponent = DatePickerComponent;
      format = 'dd/MM/yy';
    }
    return (
      <div style={{ padding: '10px 15px 0 15px' }}>
        <Grid style={{ paddingTop: 15 }} container spacing={24}>
          <Grid item xs={12}>
            <TextBoxComponent
              name="Subject"
              data-name="Subject"
              id="Subject"
              className="e-subject e-field e-input"
              placeholder="Title"
              floatLabelType="Auto"
              value={props.Subject}
            />
          </Grid>
          <Grid item xs={6}>
            <Datecomponent
              id="StartTime"
              format={format}
              data-name="StartTime"
              value={new Date(props.startTime || props.StartTime)}
              className="e-start e-field e-control e-datetimepicker e-lib e-keyboard"
            />
          </Grid>
          <Grid item xs={6}>
            <Datecomponent
              id="EndTime"
              format={format}
              data-name="EndTime"
              value={new Date(props.endTime || props.EndTime)}
              className="e-end e-field e-control e-datetimepicker e-lib e-keyboard"
            />
          </Grid>
          {/* {props.Timezone && */}
            <Grid className="e-time-zone-row" container>
              <Grid className="e-start-time-zone-container" item xs={6}>
                <DropDownListComponent
                  id="StartTimezone"
                  placeholder="Select a timezone"
                  data-name="StartTimezone"
                  className="e-start-time-zone e-field e-control e-dropdownlist e-lib"
                  style={{ width: '100%' }}
                  allowFiltering={true}
                  dataSource={this.timezoneData}
                  fields={{ text: 'Label', value: 'Value' }}
                  filtering={this.onFilteringTimezone}
                  value={props.StartTimezone || ''}
                />
              </Grid>
              <Grid className="e-end-time-zone-container" item xs={6}>
                <DropDownListComponent
                  id="EndTimezone"
                  placeholder="Select a timezone"
                  data-name="EndTimezone"
                  className="e-end-time-zone e-field e-control e-dropdownlist e-lib"
                  style={{ width: '100%' }}
                  filterBarPlaceholder='Search a timezone'
                  allowFiltering={true}
                  dataSource={this.timezoneData}
                  fields={{ text: 'Label', value: 'Value' }}
                  filtering={this.onFilteringTimezone}
                  value={props.EndTimezone || ''}
                />
              </Grid>
            </Grid>
          {/* } */}
          <Grid item xs={6}>
            <TextBoxComponent
              name="StartLocation"
              data-name="StartLocation"
              id="StartLocation"
              className="e-field e-input"
              placeholder="Start Location"
              floatLabelType="Auto"
              value={props.StartLocation || ''}
            />
          </Grid>
          <Grid item xs={6}>
            <TextBoxComponent
              name="EndLocation"
              data-name="EndLocation"
              id="EndLocation"
              className="e-field e-input"
              placeholder="End Location"
              floatLabelType="Auto"
              value={props.EndLocation || ''}
            />
          </Grid>
          <Grid
            item
            xs={12}
            style={{
              display: 'flex',
              alignItems: 'center',
              justifyContent: 'space-around'
            }}
          >
            <CheckBoxComponent
              name="AllDay"
              data-name="AllDay"
              id="AllDay"
              className="e-all-day e-field e-control e-checkbox e-lib"
              checked={props.AllDay}
              label="All Day?"
            />
            <CheckBoxComponent
              name="Timezone"
              data-name="Timezone"
              className="e-time-zone e-field e-control e-checkbox e-lib"
              cssClass="e-checkbox-wrapper e-wrapper e-time-zone"
              checked={props.Timezone}
              onClick={this.handleTimeZoneChange}
              label="Timezone"
              title="Timezone"
            />
            <ColorPickerComponent
              id='Color'
              data-name="Color"
              className="e-field"
              mode={'Palette'}
              disabled={false}
              showButtons={true}
              value={props.CategoryColor || '#5ab9de'}
              modeSwitcher={'true'}
              // change={this.onChange.bind(this)}
            />
          </Grid>
          <Grid item xs={12}>
            <RecurrenceEditorComponent
              id="RecurrenceRule"
              data-name="RecurrenceRule"
              name="RecurrenceRule"
              className="e-field"
              value={props.RecurrenceRule || ""}
            />
          </Grid>
          <Grid item xs={12}>
            <MultiSelectComponent
              id="EmployeeId"
              data-id="EmployeeId"
              className="e-field"
              dataSource={this.employeeData}
              fields={this.fields}
              mode="Box"
              placeholder="Select employee"
              itemTemplate={this.itemTemplate}
              valueTemplate={this.valueTemplate}
              value={[props.EmployeeId] || []}
              // headerTemplate={this.headerTemplate}
            />
          </Grid>
          <Grid item xs={6}>
            <DropDownListComponent
              id="Status"
              placeholder="Choose status"
              data-name="Status"
              className="e-field"
              style={{ width: '100%' }}
              dataSource={[ 'New', 'Start', 'Requested', 'Confirmed', 'Completed' ]}
              value={props.Status || ''}
            />
          </Grid>
        </Grid>
        <Grid container spacing={24}>
          <Grid item xs={6}>
            <TextBoxComponent
              name="Flight"
              data-name="Flight"
              id="Flight"
              className="e-field e-input"
              placeholder="Flight"
              floatLabelType="Auto"
              value={props.Flight || ''}
            />
          </Grid>
        </Grid>
        <button type="submit" title="submit" style={{ display: 'none' }}></button>
      </div>
    );
  };

  editNoteTab = () => {
    return (
      <div className="control-pane">
        <div className="control-section" id="rteTools">
          <div className="rte-control-section">
            <RichTextEditorComponent
              id="toolsRTE"
              ref={(richtexteditor) => {
                this.rteObj = richtexteditor;
              }}
              showCharCount={true}
              actionBegin={this.handleFullScreen.bind(this)}
              actionComplete={this.actionCompleteHandler.bind(this)}
              maxLength={2000}
              toolbarSettings={this.toolbarSettings}
            >
              <Inject services={[ Toolbar, Image, Link, HtmlEditor, Count, QuickToolbar, Table ]} />
            </RichTextEditorComponent>
          </div>
        </div>
      </div>
    );
  };

  editorTemplate = (props) => {
    return props !== undefined ? (
      <Paper square>
        <TabComponent className="custom-event-editor" style={{ width: '100%' }} id="defaultTab">
          <TabItemsDirective>
            <TabItemDirective header={this.headertext[0]} content={() => <EventEditorTemplate {...props} timezoneData={this.timezoneData} onFilteringTimezone={this.onFilteringTimezone} employeeData={this.employeeData} fields={this.fields} itemTemplate={this.itemTemplate} valueTemplate={this.valueTemplate} />} />
            <TabItemDirective header={this.headertext[1]} content={''} />
            <TabItemDirective header={this.headertext[2]} content={''} />
            <TabItemDirective header={this.headertext[3]} content={() => this.editNoteTab(props)} />
          </TabItemsDirective>
        </TabComponent>
      </Paper>
    ) : (
      <div />
    );
  };

  onEventRendered = (args) => {
    // console.log('args', args);
    let categoryColor = args.data.CategoryColor;
    if (!args.element || !categoryColor || args.data.IsBlock) {
      return;
    }
    if (this.scheduleObj.currentView === 'Agenda') {
      args.element.firstChild.style.borderLeftColor = categoryColor;
    } else {
      args.element.style.backgroundColor = categoryColor;
    }
  }

  treeTemplate = (props) => {
    console.log('props', props);
    return (
      <div className="e-text-content" id="waiting">
        <div id="waitdetails">
          <div className="e-list-text" id="waitlist">{props.Subject}</div>
          <div id="waitcategory">
            {props.Description}
          </div>
        </div>
      </div>
    );
  }

  onTreeDragStop = (event) => {
    let treeElement = closest(event.target, '.e-treeview');
    let classElement = this.scheduleObj.element.querySelector('.e-device-hover');
    if (classElement) {
      classElement.classList.remove('e-device-hover');
    }
    if (!treeElement) {
      event.cancel = true;
      let scheduleElement = closest(event.target, '.e-content-wrap');
      if (scheduleElement) {
        let treeviewData = this.treeObj.fields.dataSource;
        if (event.target.classList.contains('e-work-cells')) {
          const filteredData = treeviewData.filter(
            (item) => item.Id === parseInt(event.draggedNodeData.id, 10)
          );
          let cellData = this.scheduleObj.getCellDetails(event.target);
          let resourceDetails = this.scheduleObj.getResourcesByIndex(cellData.groupIndex);
          console.log('filteredData', filteredData);
          console.log('cellData', cellData);
          console.log('resourceDetails.resourceData', resourceDetails.resourceData);
          let eventData = {
            Subject: filteredData[0].Subject,
            StartTime: cellData.startTime,
            EndTime: cellData.endTime,
            IsAllDay: cellData.isAllDay,
            Remark: filteredData[0].Description,
            EmployeeId: [resourceDetails.resourceData.Id],
            Id: filteredData[0].Id
          };
          this.scheduleObj.openEditor(eventData, 'Add', true);
          this.isTreeItemDropped = true;
          this.draggedItemId = event.draggedNodeData.id;
        }
      }
    }
  }

  onItemDrag = (event) => {
    if (this.scheduleObj.isAdaptive) {
      let classElement = this.scheduleObj.element.querySelector('.e-device-hover');
      if (classElement) {
        classElement.classList.remove('e-device-hover');
      }
      if (event.target.classList.contains('e-work-cells')) {
        addClass([ event.target ], 'e-device-hover');
      }
    }
    if (document.body.style.cursor === 'not-allowed') {
      document.body.style.cursor = '';
    }
    if (event.name === 'nodeDragging') {
      let dragElementIcon = document.querySelectorAll('.e-drag-item.treeview-external-drag .e-icon-expandable');
      for (let i = 0; i < dragElementIcon.length; i++) {
        dragElementIcon[i].style.display = 'none';
      }
    }
  }

  onActionBegin = (args) => {
    console.log('onActionBegin args', args);
    // if (args.requestType === 'eventCreate' || args.requestType === 'eventChange') {
    //   let data = args.data instanceof Array ? args.data[0] : args.data;
    //   if (!this.scheduleObj.isSlotAvailable(data.StartTime, data.EndTime)) {
    //     args.cancel = true;
    //   }
    // }

    if (args.requestType === 'eventCreate' && this.isTreeItemDropped) {
      let treeViewdata = this.treeObj.fields.dataSource;
      const filteredPeople = treeViewdata.filter((item) => item.Id !== parseInt(this.draggedItemId, 10));
      this.treeObj.fields.dataSource = filteredPeople;
      let elements = document.querySelectorAll('.e-drag-item.treeview-external-drag');
      for (let i = 0; i < elements.length; i++) {
        remove(elements[i]);
      }
    }
  }

  // onClear = (event) => {
  //   console.log('onClear event', event);
  // };

  // onNavigating = (event) => {
  //   console.log('onNavigating event', event);
  // };

  // onDestroyed = (event) => {
  //   console.log('onDestroyed event', event);
  // };

  // onEventClick = (event) => {
  //   console.log('onEventClick event', event);
  // };

  popupOpen = (args) => {
    console.log('popupOpen args', args);
    if (args.type === 'Editor') {
      args.duration = 60;
    }
  };

  handleSubmitTaskData = (values) => {
    console.log('values', values);
  }

  render() {
    return (
      <div className="schedule-control-section">
        <div className="col-lg-12 control-section">
          <div className="control-wrapper drag-sample-wrapper">
            <div className="schedule-container">
              <div className="title-container">
                <h1 className="title-text">Stff/Personnel Task Scheduler</h1>
              </div>
              <ScheduleComponent
                ref={(schedule) => (this.scheduleObj = schedule)}
                cssClass="block-events"
                width="100%"
                height="650px"
                currentView="TimelineDay"
                resourceHeaderTemplate={this.resourceHeaderTemplate}
                group={{ enableCompactView: false, resources: [ 'Employee' ] }}
                editorTemplate={this.editorTemplate}
                eventRendered={this.onEventRendered}
                eventSettings={{
                  dataSource: this.dataManger,
                  dataQuery : new Query().from("EventDatas"),
                  fields: {
                    id: 'Id',
                    subject: 'Subject',
                    location: 'Location',
                    description: 'Remark',
                    startTime: 'StartTime',
                    endTime: 'EndTime'
                  }
                }}
                actionBegin={this.onActionBegin}
                drag={this.onItemDrag}
                popupOpen={this.popupOpen}
                // destroyed={this.onDestroyed}
                // navigating={this.onNavigating}
                // eventClick={this.onEventClick}
                // eventSettings={{ dataSource: this.state.tasks }}
                // workHours={{ start: '08:00', end: '18:00' }}
              >
                <ResourcesDirective>
                  <ResourceDirective
                    field="EmployeeId"
                    title="Employees"
                    name="Employee"
                    allowMultiple={true}
                    dataSource={this.employeeData}
                    textField="Text"
                    idField="Id"
                    colorField="Color"
                  />
                </ResourcesDirective>
                <ViewsDirective>
                  <ViewDirective option="Day" />
                  <ViewDirective option="Week" />
                  <ViewDirective option="TimelineDay" />
                  <ViewDirective option="TimelineMonth" />
                  <ViewDirective option="TimelineWeek" />
                </ViewsDirective>
                <Inject services={[ Day, Week, TimelineViews, TimelineMonth, Resize, DragAndDrop ]} />
              </ScheduleComponent>
            </div>
            <div className="treeview-container">
              <div className="title-container">
                <h1 className="title-text">Waiting List</h1>
              </div>
              <TreeViewComponent
                ref={(tree) => (this.treeObj = tree)}
                cssClass="treeview-external-drag"
                // nodeTemplate={this.treeTemplate}
                fields={this.treeViewFields}
                nodeDragStop={this.onTreeDragStop}
                nodeDragging={this.onItemDrag}
                allowDragAndDrop={this.allowDragAndDrops}
              />
              <ul className="e-ul" style={{ padding: 0, marginTop: 0 }}>
                <li className="e-list-item" style={{ margin: '0 3px', textAlign: 'center' }}>
                  <AddWaitingTaskModal handleSubmitTaskData={this.handleSubmitTaskData} />
                </li>
              </ul>
            </div>
          </div>
        </div>
      </div>
    );
  }
}

export default TaskScheduler;
